@@ -1,8 +1,8 @@
 /*****************************************************************************
  * dtstospdif.c : encapsulates DTS frames into S/PDIF packets
  *****************************************************************************
- * Copyright (C) 2003 the VideoLAN team
- * $Id: dtstospdif.c 13671 2005-12-10 19:26:03Z hartman $
+ * Copyright (C) 2003, 2006 the VideoLAN team
+ * $Id: dtstospdif.c 14997 2006-03-31 15:15:07Z fkuehne $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -67,7 +67,7 @@ static void DoWork    ( aout_instance_t *, aout_filter_t *, aout_buffer_t *,
 vlc_module_begin();
     set_category( CAT_AUDIO );
     set_subcategory( SUBCAT_AUDIO_MISC );
-    set_description( _("audio filter for DTS->S/PDIF encapsulation") );
+    set_description( _("Audio filter for DTS->S/PDIF encapsulation") );
     set_capability( "audio filter", 10 );
     set_callbacks( Create, Close );
 vlc_module_end();
