@@ -1,8 +1,8 @@
 /*****************************************************************************
  * float32tos16.c : converter from float32 to signed 16 bits integer
  *****************************************************************************
- * Copyright (C) 2002 VideoLAN
- * $Id: float32tos16.c 8391 2004-08-06 17:28:36Z sam $
+ * Copyright (C) 2002, 2006 the VideoLAN team
+ * $Id: float32tos16.c 14997 2006-03-31 15:15:07Z fkuehne $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -43,7 +43,9 @@ static void DoWork    ( aout_instance_t *, aout_filter_t *, aout_buffer_t *,
  * Module descriptor
  *****************************************************************************/
 vlc_module_begin();
-    set_description( _("audio filter for float32->s16 conversion") );
+    set_category( CAT_AUDIO );
+    set_subcategory( SUBCAT_AUDIO_MISC );
+    set_description( _("Audio filter for float32->s16 conversion") );
     set_capability( "audio filter", 1 );
     set_callbacks( Create, NULL );
 vlc_module_end();
