@@ -2,7 +2,7 @@
  * mjpeg.c : demuxes mjpeg webcam http streams
  *****************************************************************************
  * Copyright (C) 2004 the VideoLAN team
- * $Id: mjpeg.c 7196 2004-03-29 21:29:31Z fenrir $
+ * $Id: mjpeg.c 14790 2006-03-18 02:06:16Z xtophe $
  *
  * Authors: Henry Jen (slowhog) <henryjen@ztune.net>
  *          Derk-Jan Hartman (thedj)
@@ -21,7 +21,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -41,12 +41,13 @@ static int  Open ( vlc_object_t * );
 static void Close( vlc_object_t * );
 
 #define FPS_TEXT N_("Frames per Second")
-#define FPS_LONGTEXT N_("Allows you to set the desired frame rate when " \
-    "playing from files, use 0 for live.")
+#define FPS_LONGTEXT N_("This is the desired frame rate when " \
+    "playing MJPEG from a file. Use 0 (this is the default value) for a " \
+    "live stream (from a camera).")
 
 vlc_module_begin();
     set_shortname( "MJPEG");
-    set_description( _("JPEG camera demuxer") );
+    set_description( _("M-JPEG camera demuxer") );
     set_capability( "demux2", 5 );
     set_callbacks( Open, Close );
     set_category( CAT_INPUT );
@@ -235,7 +236,7 @@ static vlc_bool_t CheckMimeHeader( demux_t *p_demux, int *p_header_size )
         }
         else
         {
-            msg_Dbg( p_demux, "Discard MIME header: %s", psz_line );
+            msg_Dbg( p_demux, "discard MIME header: %s", psz_line );
         }
         free( psz_line );
         psz_line = GetLine( p_demux, &i_pos );
@@ -407,11 +408,11 @@ static int MjpgDemux( demux_t *p_demux )
         i++;
         if( i >= p_sys->i_data_peeked )
         {
-            msg_Dbg( p_demux, "Did not find JPEG EOI in %d bytes",
+            msg_Dbg( p_demux, "did not find JPEG EOI in %d bytes",
                      p_sys->i_data_peeked );
             if( !Peek( p_demux, VLC_FALSE ) )
             {
-                msg_Warn( p_demux, "No more data is available at the moment" );
+                msg_Warn( p_demux, "no more data is available at the moment" );
                 return 0;
             }
         }
@@ -470,7 +471,7 @@ static int MimeDemux( demux_t *p_demux )
 
                 if( !Peek( p_demux, VLC_FALSE ) )
                 {
-                    msg_Warn( p_demux, "No more data is available at the "
+                    msg_Warn( p_demux, "no more data is available at the "
                               "moment" );
                     return 0;
                 }
@@ -490,7 +491,7 @@ static int MimeDemux( demux_t *p_demux )
 
     if( !b_match )
     {
-        msg_Err( p_demux, "Discard non-JPEG part" );
+        msg_Err( p_demux, "discard non-JPEG part" );
         stream_Read( p_demux->s, NULL, i );
         return 0;
     }
