@@ -1,8 +1,8 @@
 /*****************************************************************************
  * mpga.c : MPEG-I/II Audio input module for vlc
  *****************************************************************************
- * Copyright (C) 2001-2004 VideoLAN
- * $Id: mpga.c 10615 2005-04-09 11:27:23Z gbazin $
+ * Copyright (C) 2001-2004 the VideoLAN team
+ * $Id: mpga.c 14923 2006-03-25 15:39:09Z zorglub $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -32,7 +32,7 @@
 #include "vlc_codec.h"
 #include "vlc_meta.h"
 
-#define MPGA_PACKET_SIZE 4096
+#define MPGA_PACKET_SIZE 1024
 
 /*****************************************************************************
  * Module descriptor
@@ -43,7 +43,7 @@ static void Close( vlc_object_t * );
 vlc_module_begin();
     set_category( CAT_INPUT );
     set_subcategory( SUBCAT_INPUT_DEMUX );
-    set_description( _("MPEG-I/II audio demuxer" ) );
+    set_description( _("MPEG audio / MP3 demuxer" ) );
     set_capability( "demux2", 100 );
     set_callbacks( Open, Close );
     add_shortcut( "mpga" );
@@ -68,6 +68,8 @@ struct demux_sys_t
     mtime_t     i_time_offset;
     int         i_bitrate_avg;  /* extracted from Xing header */
 
+    vlc_bool_t b_initial_sync_failed;
+
     int i_xing_frames;
     int i_xing_bytes;
     int i_xing_bitrate_avg;
@@ -108,6 +110,7 @@ static int mpga_frame_samples( uint32_t h )
     }
 }
 
+
 /*****************************************************************************
  * Open: initializes demux structures
  *****************************************************************************/
@@ -202,7 +205,7 @@ static int Open( vlc_object_t * p_this )
             i_skip = MPGA_MODE( header ) != 3 ? 21 : 13;
         }
 
-        if( i_skip + 8 < i_xing && !strncmp( &p_xing[i_skip], "Xing", 4 ) )
+        if( i_skip + 8 < i_xing && !strncmp( (char *)&p_xing[i_skip], "Xing", 4 ) )
         {
             unsigned int i_flags = GetDWBE( &p_xing[i_skip+4] );
 
@@ -252,12 +255,20 @@ static int Open( vlc_object_t * p_this )
     p_block_in->i_pts = p_block_in->i_dts = 1;
     p_block_out = p_sys->p_packetizer->pf_packetize(
         p_sys->p_packetizer, &p_block_in );
-    
+
+    if( p_block_out == NULL )
+    {
+        msg_Dbg( p_demux, "did not sync on first block" );
+        p_sys->b_initial_sync_failed = VLC_TRUE;
+    }
+    else
+        p_sys->b_initial_sync_failed = VLC_FALSE;
+
     p_sys->p_packetizer->fmt_out.b_packetized = VLC_TRUE;
     p_sys->p_es = es_out_Add( p_demux->out,
                               &p_sys->p_packetizer->fmt_out);
     p_sys->i_bitrate_avg = p_sys->p_packetizer->fmt_out.i_bitrate;
-    
+
     if( p_sys->i_xing_bytes && p_sys->i_xing_frames &&
         p_sys->i_xing_frame_samples )
     {
@@ -302,7 +313,14 @@ static int Demux( demux_t *p_demux )
         {
             return 0;
         }
-        p_block_in->i_pts = p_block_in->i_dts = 0;
+        if( p_demux->p_sys->b_initial_sync_failed == VLC_TRUE )
+        {
+            p_block_in->i_pts = p_block_in->i_dts = 1;
+            /* Only try to resync once */
+            p_demux->p_sys->b_initial_sync_failed = 0;
+        }
+        else
+            p_block_in->i_pts = p_block_in->i_dts = 0;
         p_block_out = p_sys->p_packetizer->pf_packetize(
             p_sys->p_packetizer, &p_block_in );
     }
