@@ -2,10 +2,10 @@
  * ctrl_generic.cpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: ctrl_generic.cpp 11664 2005-07-09 06:17:09Z courmisch $
+ * $Id: ctrl_generic.cpp 14118 2006-02-01 18:06:48Z courmisch $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
- *          Olivier Teulire <ipkiss@via.ecp.fr>
+ *          Olivier Teulière <ipkiss@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #include "ctrl_generic.hpp"
@@ -80,8 +80,7 @@ void CtrlGeneric::notifyLayout( int width, int height,
 }
 
 
-void CtrlGeneric::notifyLayoutMaxSize( const OSGraphics *pImg1,
-                                       const OSGraphics *pImg2 )
+void CtrlGeneric::notifyLayoutMaxSize( const Box *pImg1, const Box *pImg2 )
 {
     if( pImg1 == NULL )
     {
@@ -156,7 +155,7 @@ bool CtrlGeneric::isVisible() const
 }
 
 
-void CtrlGeneric::onUpdate( Subject<VarBool> &rVariable )
+void CtrlGeneric::onUpdate( Subject<VarBool, void*> &rVariable, void *arg  )
 {
     // Is it the visibility variable ?
     if( &rVariable == m_pVisible )
