@@ -2,10 +2,10 @@
  * evt_special.cpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: evt_special.cpp 11664 2005-07-09 06:17:09Z courmisch $
+ * $Id: evt_special.cpp 15008 2006-03-31 19:24:33Z zorglub $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
- *          Olivier Teulire <ipkiss@via.ecp.fr>
+ *          Olivier Teulière <ipkiss@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #include "evt_special.hpp"
@@ -39,7 +39,7 @@ const string EvtSpecial::getAsString() const
     else if( m_action == kDisable )
         event += ":disable";
     else
-        msg_Warn( getIntf(), "Unknown action type" );
+        msg_Warn( getIntf(), "unknown action type" );
 
     return event;
 }
