@@ -1,11 +1,11 @@
 /*****************************************************************************
  * x11_factory.cpp
  *****************************************************************************
- * Copyright (C) 2003 VideoLAN
- * $Id: x11_factory.cpp 7335 2004-04-12 21:48:18Z gbazin $
+ * Copyright (C) 2003 the VideoLAN team
+ * $Id: x11_factory.cpp 14118 2006-02-01 18:06:48Z courmisch $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
- *          Olivier Teulire <ipkiss@via.ecp.fr>
+ *          Olivier Teulière <ipkiss@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #ifdef X11_SKINS
@@ -33,6 +33,7 @@
 #include "x11_display.hpp"
 #include "x11_graphics.hpp"
 #include "x11_loop.hpp"
+#include "x11_popup.hpp"
 #include "x11_timer.hpp"
 #include "x11_window.hpp"
 #include "x11_tooltip.hpp"
@@ -96,10 +97,15 @@ void X11Factory::destroyOSLoop()
     X11Loop::destroy( getIntf() );
 }
 
-
-OSTimer *X11Factory::createOSTimer( const Callback &rCallback )
+void X11Factory::minimize()
 {
-    return new X11Timer( getIntf(), rCallback );
+    XIconifyWindow( m_pDisplay->getDisplay(), m_pDisplay->getMainWindow(),
+                    DefaultScreen( m_pDisplay->getDisplay() ) );
+}
+
+OSTimer *X11Factory::createOSTimer( CmdGeneric &rCmd )
+{
+    return new X11Timer( getIntf(), rCmd );
 }
 
 
@@ -117,6 +123,12 @@ OSTooltip *X11Factory::createOSTooltip()
 }
 
 
+OSPopup *X11Factory::createOSPopup()
+{
+    return new X11Popup( getIntf(), *m_pDisplay );
+}
+
+
 int X11Factory::getScreenWidth() const
 {
     Display *pDisplay = m_pDisplay->getDisplay();
