@@ -1,12 +1,13 @@
 /*****************************************************************************
  * ipv4.c: IPv4 network abstraction layer
  *****************************************************************************
- * Copyright (C) 2001, 2002 VideoLAN
- * $Id: ipv4.c 11387 2005-06-10 15:32:08Z hartman $
+ * Copyright (C) 2001-2006 the VideoLAN team
+ * $Id: ipv4.c 14932 2006-03-25 23:10:43Z xtophe $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Mathias Kretschmer <mathias@research.att.com>
  *          Alexis de Lattre <alexis@via.ecp.fr>
+ *          Rémi Denis-Courmont <rem # videolan.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -52,6 +53,7 @@
 #   endif
 #   include <winsock2.h>
 #   include <ws2tcpip.h>
+#   include <iphlpapi.h>
 #   define close closesocket
 #   if defined(UNDER_CE)
 #       undef IP_MULTICAST_TTL
@@ -79,106 +81,93 @@
 #ifndef IN_MULTICAST
 #   define IN_MULTICAST(a) IN_CLASSD(a)
 #endif
-#ifndef PF_INET
-#    define PF_INET AF_INET                                          /* BeOS */
-#endif
 
 
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
-static int NetOpen( vlc_object_t * );
+static int OpenUDP( vlc_object_t * );
 
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-#define TIMEOUT_TEXT N_("TCP connection timeout in ms")
-#define TIMEOUT_LONGTEXT N_( \
-    "Allows you to modify the default TCP connection timeout. This " \
-    "value should be set in millisecond units." )
-
-#define MIFACE_TEXT N_("Multicast output interface")
-#define MIFACE_LONGTEXT N_( \
-    "Indicate here the multicast output interface. " \
-    "This overrides the routing table.")
-
 vlc_module_begin();
-    set_description( _("IPv4 network abstraction layer") );
+    set_shortname( "IPv4" );
+    set_description( _("UDP/IPv4 network abstraction layer") );
     set_capability( "network", 50 );
     set_category( CAT_INPUT );
-    set_subcategory( SUBCAT_INPUT_ADVANCED );
-    set_callbacks( NetOpen, NULL );
-
-    add_integer( "ipv4-timeout", 5 * 1000, NULL, TIMEOUT_TEXT,
-                 TIMEOUT_LONGTEXT, VLC_TRUE );
-    add_string( "miface-addr", NULL, NULL, MIFACE_TEXT, MIFACE_LONGTEXT, VLC_TRUE );
+    set_subcategory( SUBCAT_INPUT_GENERAL );
+    set_callbacks( OpenUDP, NULL );
 vlc_module_end();
 
 /*****************************************************************************
  * BuildAddr: utility function to build a struct sockaddr_in
  *****************************************************************************/
-static int BuildAddr( struct sockaddr_in * p_socket,
+static int BuildAddr( vlc_object_t *p_obj, struct sockaddr_in * p_socket,
                       const char * psz_address, int i_port )
 {
-    /* Reset struct */
-    memset( p_socket, 0, sizeof( struct sockaddr_in ) );
-    p_socket->sin_family = AF_INET;                                /* family */
-    p_socket->sin_port = htons( (uint16_t)i_port );
-    if( !*psz_address )
-    {
-        p_socket->sin_addr.s_addr = INADDR_ANY;
-    }
-    else
-    {
-        struct hostent    * p_hostent;
+    struct addrinfo hints, *res;
+    int i_val;
 
-        /* Try to convert address directly from in_addr - this will work if
-         * psz_address is dotted decimal. */
-#ifdef HAVE_ARPA_INET_H
-        if( !inet_aton( psz_address, &p_socket->sin_addr ) )
-#else
-        p_socket->sin_addr.s_addr = inet_addr( psz_address );
-        if( p_socket->sin_addr.s_addr == INADDR_NONE )
-#endif
-        {
-            /* We have a fqdn, try to find its address */
-            if ( (p_hostent = gethostbyname( psz_address )) == NULL )
-            {
-                return( -1 );
-            }
+    memset( &hints, 0, sizeof( hints ) );
+    hints.ai_family = AF_INET;
+    hints.ai_socktype = SOCK_DGRAM;
+    hints.ai_flags = AI_PASSIVE;
 
-            /* Copy the first address of the host in the socket address */
-            memcpy( &p_socket->sin_addr, p_hostent->h_addr_list[0],
-                     p_hostent->h_length );
-        }
+    msg_Dbg( p_obj, "resolving %s:%d...", psz_address, i_port );
+    i_val = vlc_getaddrinfo( p_obj, psz_address, i_port, &hints, &res );
+    if( i_val )
+    {
+        msg_Warn( p_obj, "%s: %s", psz_address, vlc_gai_strerror( i_val ) );
+        return -1;
     }
+
+    /* Copy the first address of the host in the socket address */
+    memcpy( p_socket, res->ai_addr, sizeof( *p_socket ) );
+    vlc_freeaddrinfo( res );
+
     return( 0 );
 }
 
+#if defined(WIN32) || defined(UNDER_CE)
+# define WINSOCK_STRERROR_SIZE 20
+static const char *winsock_strerror( char *buf )
+{
+    snprintf( buf, WINSOCK_STRERROR_SIZE, "Winsock error %d",
+              WSAGetLastError( ) );
+    buf[WINSOCK_STRERROR_SIZE - 1] = '\0';
+    return buf;
+}
+#endif
+
+
 /*****************************************************************************
  * OpenUDP: open a UDP socket
  *****************************************************************************
  * psz_bind_addr, i_bind_port : address and port used for the bind()
  *   system call. If psz_bind_addr == "", the socket is bound to
- *   INADDR_ANY and broadcast reception is enabled. If i_bind_port == 0,
- *   1234 is used. If psz_bind_addr is a multicast (class D) address,
- *   join the multicast group.
+ *   INADDR_ANY and broadcast reception is enabled. If psz_bind_addr is a
+ *   multicast (class D) address, join the multicast group.
  * psz_server_addr, i_server_port : address and port used for the connect()
  *   system call. It can avoid receiving packets from unauthorized IPs.
  *   Its use leads to great confusion and is currently discouraged.
  * This function returns -1 in case of error.
  *****************************************************************************/
-static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
+static int OpenUDP( vlc_object_t * p_this )
 {
-    char * psz_bind_addr = p_socket->psz_bind_addr;
+    network_socket_t * p_socket = p_this->p_private;
+    const char * psz_bind_addr = p_socket->psz_bind_addr;
     int i_bind_port = p_socket->i_bind_port;
-    char * psz_server_addr = p_socket->psz_server_addr;
+    const char * psz_server_addr = p_socket->psz_server_addr;
     int i_server_port = p_socket->i_server_port;
 
     int i_handle, i_opt;
-    socklen_t i_opt_size;
     struct sockaddr_in sock;
     vlc_value_t val;
+#if defined(WIN32) || defined(UNDER_CE)
+    char strerror_buf[WINSOCK_STRERROR_SIZE];
+# define strerror( x ) winsock_strerror( strerror_buf )
+#endif
 
     /* If IP_ADD_SOURCE_MEMBERSHIP is not defined in the headers
        (because it's not in glibc for example), we have to define the
@@ -192,16 +181,14 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
      };
 #endif
 
+    p_socket->i_handle = -1;
+
     /* Open a SOCK_DGRAM (UDP) socket, in the AF_INET domain, automatic (0)
      * protocol */
     if( (i_handle = socket( AF_INET, SOCK_DGRAM, 0 )) == -1 )
     {
-#if defined(WIN32) || defined(UNDER_CE)
-        msg_Warn( p_this, "cannot create socket (%i)", WSAGetLastError() );
-#else
         msg_Warn( p_this, "cannot create socket (%s)", strerror(errno) );
-#endif
-        return( -1 );
+        return 0;
     }
 
     /* We may want to reuse an already used socket */
@@ -209,15 +196,10 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
     if( setsockopt( i_handle, SOL_SOCKET, SO_REUSEADDR,
                     (void *) &i_opt, sizeof( i_opt ) ) == -1 )
     {
-#if defined(WIN32) || defined(UNDER_CE)
-        msg_Warn( p_this, "cannot configure socket (SO_REUSEADDR: %i)",
-                  WSAGetLastError() );
-#else
         msg_Warn( p_this, "cannot configure socket (SO_REUSEADDR: %s)",
                           strerror(errno));
-#endif
         close( i_handle );
-        return( -1 );
+        return 0;
     }
 
 #ifdef SO_REUSEPORT
@@ -231,81 +213,53 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
 
     /* Increase the receive buffer size to 1/2MB (8Mb/s during 1/2s) to avoid
      * packet loss caused by scheduling problems */
-    i_opt = 0x80000;
 #if !defined( SYS_BEOS )
-    if( setsockopt( i_handle, SOL_SOCKET, SO_RCVBUF, (void *) &i_opt, sizeof( i_opt ) ) == -1 )
-    {
-#if defined(WIN32) || defined(UNDER_CE)
-        msg_Dbg( p_this, "cannot configure socket (SO_RCVBUF: %i)",
-                 WSAGetLastError() );
-#else
+    i_opt = 0x80000;
+    if( setsockopt( i_handle, SOL_SOCKET, SO_RCVBUF, (void *) &i_opt,
+                    sizeof( i_opt ) ) == -1 )
         msg_Dbg( p_this, "cannot configure socket (SO_RCVBUF: %s)",
                           strerror(errno));
+    i_opt = 0x80000;
+    if( setsockopt( i_handle, SOL_SOCKET, SO_SNDBUF, (void *) &i_opt,
+                    sizeof( i_opt ) ) == -1 )
+        msg_Dbg( p_this, "cannot configure socket (SO_SNDBUF: %s)",
+                          strerror(errno));
 #endif
-    }
-#endif
-
-#if !defined( SYS_BEOS )
-    /* Check if we really got what we have asked for, because Linux, etc.
-     * will silently limit the max buffer size to net.core.rmem_max which
-     * is typically only 65535 bytes */
-    i_opt = 0;
-    i_opt_size = sizeof( i_opt );
-    if( getsockopt( i_handle, SOL_SOCKET, SO_RCVBUF, (void*) &i_opt, &i_opt_size ) == -1 )
-    {
-#if defined(WIN32) || defined(UNDER_CE)
-        msg_Warn( p_this, "cannot query socket (SO_RCVBUF: %i)",
-                  WSAGetLastError() );
-#else
-        msg_Warn( p_this, "cannot query socket (SO_RCVBUF: %s)",
-                          strerror(errno) );
-#endif
-    }
-    else if( i_opt < 0x80000 )
-    {
-        msg_Dbg( p_this, "socket buffer size is 0x%x instead of 0x%x",
-                         i_opt, 0x80000 );
-    }
-#endif
-
 
     /* Build the local socket */
 
 #if defined( WIN32 ) || defined( UNDER_CE )
     /* Under Win32 and for multicasting, we bind to INADDR_ANY,
      * so let's call BuildAddr with "" instead of psz_bind_addr */
-    if( BuildAddr( &sock, IN_MULTICAST( ntohl( inet_addr(psz_bind_addr) ) ) ?
+    if( BuildAddr( p_this, &sock,
+        IN_MULTICAST( ntohl( inet_addr(psz_bind_addr) ) ) ?
                    "" : psz_bind_addr, i_bind_port ) == -1 )
 #else
-    if( BuildAddr( &sock, psz_bind_addr, i_bind_port ) == -1 )
+    if( BuildAddr( p_this, &sock, psz_bind_addr, i_bind_port ) == -1 )
 #endif
     {
         msg_Dbg( p_this, "could not build local address" );
         close( i_handle );
-        return( -1 );
+        return 0;
     }
 
     /* Bind it */
     if( bind( i_handle, (struct sockaddr *)&sock, sizeof( sock ) ) < 0 )
     {
-#if defined(WIN32) || defined(UNDER_CE)
-        msg_Warn( p_this, "cannot bind socket (%i)", WSAGetLastError() );
-#else
         msg_Warn( p_this, "cannot bind socket (%s)", strerror(errno) );
-#endif
         close( i_handle );
-        return( -1 );
+        return 0;
     }
 
 #if defined( WIN32 ) || defined( UNDER_CE )
     /* Restore the sock struct so we can spare a few #ifdef WIN32 later on */
     if( IN_MULTICAST( ntohl( inet_addr(psz_bind_addr) ) ) )
     {
-        if ( BuildAddr( &sock, psz_bind_addr, i_bind_port ) == -1 )
+        if ( BuildAddr( p_this, &sock, psz_bind_addr, i_bind_port ) == -1 )
         {
             msg_Dbg( p_this, "could not build local address" );
             close( i_handle );
-            return( -1 );
+            return 0;
         }
     }
 #endif
@@ -315,16 +269,10 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
     if( !*psz_bind_addr )
     {
         i_opt = 1;
-        if( setsockopt( i_handle, SOL_SOCKET, SO_BROADCAST, (void*) &i_opt, sizeof( i_opt ) ) == -1 )
-        {
-#if defined(WIN32) || defined(UNDER_CE)
-            msg_Warn( p_this, "cannot configure socket (SO_BROADCAST: %i)",
-                      WSAGetLastError() );
-#else
+        if( setsockopt( i_handle, SOL_SOCKET, SO_BROADCAST, (void*) &i_opt,
+                        sizeof( i_opt ) ) == -1 )
             msg_Warn( p_this, "cannot configure socket (SO_BROADCAST: %s)",
                        strerror(errno) );
-#endif
-        }
     }
 #endif
 
@@ -333,7 +281,7 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
     if( IN_MULTICAST( ntohl(sock.sin_addr.s_addr) ) )
     {
         /* Determine interface to be used for multicast */
-        char * psz_if_addr = config_GetPsz( p_this, "iface-addr" );
+        char * psz_if_addr = config_GetPsz( p_this, "miface-addr" );
 
         /* If we have a source address, we use IP_ADD_SOURCE_MEMBERSHIP
            so that IGMPv3 aware OSes running on IGMPv3 aware networks
@@ -362,16 +310,11 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
                          (char*)&imr,
                          sizeof(struct ip_mreq_source) ) == -1 )
             {
-#if defined(WIN32) || defined(UNDER_CE)
-                msg_Err( p_this, "failed to join IP multicast group (%i)",
-                         WSAGetLastError() );
-#else
                 msg_Err( p_this, "failed to join IP multicast group (%s)",
                                   strerror(errno) );
-#endif
                 msg_Err( p_this, "are you sure your OS supports IGMPv3?" );
                 close( i_handle );
-                return( -1 );
+                return 0;
             }
          }
          /* If there is no source address, we use IP_ADD_MEMBERSHIP */
@@ -379,16 +322,70 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
          {
              struct ip_mreq imr;
 
+             imr.imr_interface.s_addr = INADDR_ANY;
              imr.imr_multiaddr.s_addr = sock.sin_addr.s_addr;
              if( psz_if_addr != NULL && *psz_if_addr
                 && inet_addr(psz_if_addr) != INADDR_NONE )
             {
                 imr.imr_interface.s_addr = inet_addr(psz_if_addr);
             }
+#if defined (WIN32) || defined (UNDER_CE)
             else
             {
-                imr.imr_interface.s_addr = INADDR_ANY;
+                typedef DWORD (CALLBACK * GETBESTINTERFACE) ( IPAddr, PDWORD );
+                typedef DWORD (CALLBACK * GETIPADDRTABLE) ( PMIB_IPADDRTABLE, PULONG, BOOL );
+
+                GETBESTINTERFACE OurGetBestInterface;
+                GETIPADDRTABLE OurGetIpAddrTable;
+                HINSTANCE hiphlpapi = LoadLibrary(_T("Iphlpapi.dll"));
+                DWORD i_index;
+
+                if( hiphlpapi )
+                {
+                    OurGetBestInterface =
+                        (void *)GetProcAddress( hiphlpapi,
+                                                _T("GetBestInterface") );
+                    OurGetIpAddrTable =
+                        (void *)GetProcAddress( hiphlpapi,
+                                                _T("GetIpAddrTable") );
+                }
+
+                if( hiphlpapi && OurGetBestInterface && OurGetIpAddrTable &&
+                    OurGetBestInterface( sock.sin_addr.s_addr,
+                                         &i_index ) == NO_ERROR )
+                {
+                    PMIB_IPADDRTABLE p_table;
+                    DWORD i = 0;
+
+                    msg_Dbg( p_this, "Winsock best interface is %lu",
+                             (unsigned long)i_index );
+                    OurGetIpAddrTable( NULL, &i, 0 );
+
+                    p_table = (PMIB_IPADDRTABLE)malloc( i );
+                    if( p_table != NULL )
+                    {
+                        if( OurGetIpAddrTable( p_table, &i, 0 ) == NO_ERROR )
+                        {
+                            for( i = 0; i < p_table->dwNumEntries; i-- )
+                            {
+                                if( p_table->table[i].dwIndex == i_index )
+                                {
+                                    imr.imr_interface.s_addr =
+                                                     p_table->table[i].dwAddr;
+                                    msg_Dbg( p_this, "using interface 0x%08x",
+                                             p_table->table[i].dwAddr );
+                                }
+                            }
+                        }
+                        else msg_Warn( p_this, "GetIpAddrTable failed" );
+                        free( p_table );
+                    }
+                }
+                else msg_Dbg( p_this, "GetBestInterface failed" );
+
+                if( hiphlpapi ) FreeLibrary( hiphlpapi );
             }
+#endif
             if( psz_if_addr != NULL ) free( psz_if_addr );
 
             msg_Dbg( p_this, "IP_ADD_MEMBERSHIP multicast request" );
@@ -396,41 +393,32 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
             if( setsockopt( i_handle, IPPROTO_IP, IP_ADD_MEMBERSHIP,
                             (char*)&imr, sizeof(struct ip_mreq) ) == -1 )
             {
-#if defined(WIN32) || defined(UNDER_CE)
-                msg_Err( p_this, "failed to join IP multicast group (%i)",
-                         WSAGetLastError() );
-#else
                 msg_Err( p_this, "failed to join IP multicast group (%s)",
                                   strerror(errno) );
-#endif
                 close( i_handle );
-                return( -1 );
+                return 0;
             }
          }
     }
+    else
 #endif
-
     if( *psz_server_addr )
     {
         /* Build socket for remote connection */
-        if ( BuildAddr( &sock, psz_server_addr, i_server_port ) == -1 )
+        if ( BuildAddr( p_this, &sock, psz_server_addr, i_server_port ) == -1 )
         {
             msg_Warn( p_this, "cannot build remote address" );
             close( i_handle );
-            return( -1 );
+            return 0;
         }
 
         /* Connect the socket */
         if( connect( i_handle, (struct sockaddr *) &sock,
                      sizeof( sock ) ) == (-1) )
         {
-#if defined(WIN32) || defined(UNDER_CE)
-            msg_Warn( p_this, "cannot connect socket (%i)", WSAGetLastError());
-#else
             msg_Warn( p_this, "cannot connect socket (%s)", strerror(errno) );
-#endif
             close( i_handle );
-            return( -1 );
+            return 0;
         }
 
 #if !defined( SYS_BEOS )
@@ -438,8 +426,7 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
         {
             /* set the time-to-live */
             int i_ttl = p_socket->i_ttl;
-            unsigned char ttl;
-            
+
             /* set the multicast interface */
             char * psz_mif_addr = config_GetPsz( p_this, "miface-addr" );
             if( psz_mif_addr )
@@ -453,39 +440,34 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
                 {
                     msg_Dbg( p_this, "failed to set multicast interface (%s).", strerror(errno) );
                     close( i_handle );
-                    return ( -1 );
+                    return 0;
                 }
             }
 
-            if( i_ttl < 1 )
-            {
-                if( var_Get( p_this, "ttl", &val ) != VLC_SUCCESS )
-                {
-                    var_Create( p_this, "ttl",
-                                VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
-                    var_Get( p_this, "ttl", &val );
-                }
-                i_ttl = val.i_int;
-            }
-            if( i_ttl < 1 ) i_ttl = 1;
-            ttl = (unsigned char) i_ttl;
+            if( i_ttl <= 0 )
+                i_ttl = config_GetInt( p_this, "ttl" );
 
-            /* There is some confusion in the world whether IP_MULTICAST_TTL 
-             * takes a byte or an int as an argument.
-             * BSD seems to indicate byte so we are going with that and use
-             * int as a fallback to be safe */
-            if( setsockopt( i_handle, IPPROTO_IP, IP_MULTICAST_TTL,
-                            &ttl, sizeof( ttl ) ) < 0 )
+            if( i_ttl > 0 )
             {
-                msg_Dbg( p_this, "failed to set ttl (%s). Let's try it "
-                         "the integer way.", strerror(errno) );
+                unsigned char ttl = (unsigned char) i_ttl;
+
+                /* There is some confusion in the world whether IP_MULTICAST_TTL 
+                * takes a byte or an int as an argument.
+                * BSD seems to indicate byte so we are going with that and use
+                * int as a fallback to be safe */
                 if( setsockopt( i_handle, IPPROTO_IP, IP_MULTICAST_TTL,
-                                &i_ttl, sizeof( i_ttl ) ) <0 )
+                                &ttl, sizeof( ttl ) ) < 0 )
                 {
-                    msg_Err( p_this, "failed to set ttl (%s)",
-                             strerror(errno) );
-                    close( i_handle );
-                    return( -1 );
+                    msg_Dbg( p_this, "failed to set ttl (%s). Let's try it "
+                            "the integer way.", strerror(errno) );
+                    if( setsockopt( i_handle, IPPROTO_IP, IP_MULTICAST_TTL,
+                                    &i_ttl, sizeof( i_ttl ) ) <0 )
+                    {
+                        msg_Err( p_this, "failed to set ttl (%s)",
+                                strerror(errno) );
+                        close( i_handle );
+                        return 0;
+                    }
                 }
             }
         }
@@ -500,250 +482,6 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
         var_Get( p_this, "mtu", &val );
     }
     p_socket->i_mtu = val.i_int;
-    return( 0 );
-}
 
-/*****************************************************************************
- * SocketTCP: create a TCP socket
- *****************************************************************************
- * This function returns -1 in case of error.
- *****************************************************************************/
-static int SocketTCP( vlc_object_t * p_this )
-{
-    int i_handle;
-    
-    /* Open a SOCK_STREAM (TCP) socket, in the PF_INET domain, automatic (0)
-     * protocol */
-    if( (i_handle = socket( PF_INET, SOCK_STREAM, 0 )) == -1 )
-    {
-#if defined(WIN32) || defined(UNDER_CE)
-        msg_Warn( p_this, "cannot create socket (%i)", WSAGetLastError() );
-#else
-        msg_Warn( p_this, "cannot create socket (%s)", strerror(errno) );
-#endif
-        return -1;
-    }
-
-    /* Set to non-blocking */
-#if defined( WIN32 ) || defined( UNDER_CE )
-    {
-        unsigned long i_dummy = 1;
-        if( ioctlsocket( i_handle, FIONBIO, &i_dummy ) != 0 )
-        {
-            msg_Err( p_this, "cannot set socket to non-blocking mode" );
-        }
-    }
-#else
-    {
-        int i_flags;
-        if( ( i_flags = fcntl( i_handle, F_GETFL, 0 ) ) < 0 ||
-            fcntl( i_handle, F_SETFL, i_flags | O_NONBLOCK ) < 0 )
-        {
-            msg_Err( p_this, "cannot set socket to non-blocking mode" );
-        }
-    }
-#endif
-
-    return i_handle;
-}
-
-/*****************************************************************************
- * OpenTCP: open a TCP socket
- *****************************************************************************
- * psz_server_addr, i_server_port : address and port used for the connect()
- *   system call. If i_server_port == 0, 80 is used.
- * Other parameters are ignored.
- * This function returns -1 in case of error.
- *****************************************************************************/
-static int OpenTCP( vlc_object_t * p_this, network_socket_t * p_socket )
-{
-    char * psz_server_addr = p_socket->psz_server_addr;
-    int i_server_port = p_socket->i_server_port;
-
-    int i_handle;
-    struct sockaddr_in sock;
-
-    if( i_server_port == 0 )
-    {
-        i_server_port = 80;
-    }
-
-    if( (i_handle = SocketTCP( p_this )) == -1 )
-        return VLC_EGENERIC;
-
-    /* Build remote address */
-    if ( BuildAddr( &sock, psz_server_addr, i_server_port ) == -1 )
-    {
-        msg_Dbg( p_this, "could not build local address" );
-        goto error;
-    }
-
-    /* Connect the socket */
-    if( connect( i_handle, (struct sockaddr *) &sock, sizeof( sock ) ) == -1 )
-    {
-#if defined( WIN32 ) || defined( UNDER_CE )
-        if( WSAGetLastError() == WSAEWOULDBLOCK )
-#else
-        if( errno == EINPROGRESS )
-#endif
-        {
-            int i_ret, i_opt, i_opt_size = sizeof( i_opt ), i_max_count;
-            struct timeval timeout;
-            vlc_value_t val;
-            fd_set fds;
-
-            if( var_Get( p_this, "ipv4-timeout", &val ) != VLC_SUCCESS )
-            {
-                var_Create( p_this, "ipv4-timeout",
-                            VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
-                var_Get( p_this, "ipv4-timeout", &val );
-            }
-            i_max_count = val.i_int * 1000 / 100000 /* timeout.tv_usec */;
-
-            msg_Dbg( p_this, "connection in progress" );
-            do
-            {
-                if( p_this->b_die || i_max_count <= 0 )
-                {
-                    msg_Dbg( p_this, "connection aborted" );
-                    goto error;
-                }
-
-                i_max_count--;
-
-                /* Initialize file descriptor set */
-                FD_ZERO( &fds );
-                FD_SET( i_handle, &fds );
-
-                /* We'll wait 0.1 second if nothing happens */
-                timeout.tv_sec = 0;
-                timeout.tv_usec = 100000;
-
-            } while( ( i_ret = select( i_handle + 1, NULL, &fds, NULL,
-                                       &timeout ) ) == 0 ||
-#if defined( WIN32 ) || defined( UNDER_CE )
-                     ( i_ret < 0 && WSAGetLastError() == WSAEWOULDBLOCK ) );
-#else
-                     ( i_ret < 0 && errno == EINTR ) );
-#endif
-
-            if( i_ret < 0 )
-            {
-                msg_Warn( p_this, "cannot connect socket (select failed)" );
-                goto error;
-            }
-
-#if !defined( SYS_BEOS ) && !defined( UNDER_CE )
-            if( getsockopt( i_handle, SOL_SOCKET, SO_ERROR, (void*)&i_opt,
-                            &i_opt_size ) == -1 || i_opt != 0 )
-            {
-                msg_Warn( p_this, "cannot connect socket (SO_ERROR)" );
-                goto error;
-            }
-#endif
-        }
-        else
-        {
-#if defined(WIN32) || defined(UNDER_CE)
-            msg_Warn( p_this, "cannot connect socket (%i)", WSAGetLastError());
-#else
-            msg_Warn( p_this, "cannot connect socket (%s)", strerror(errno) );
-#endif
-            goto error;
-        }
-    }
-
-    p_socket->i_handle = i_handle;
-    p_socket->i_mtu = 0; /* There is no MTU notion in TCP */
-    return VLC_SUCCESS;
-
-error:
-    close( i_handle );
-    return VLC_EGENERIC;
-}
-
-/*****************************************************************************
- * ListenTCP: open a TCP passive socket (server-side)
- *****************************************************************************
- * psz_server_addr, i_server_port : address and port used for the bind()
- *   system call. If i_server_port == 0, 80 is used.
- * Other parameters are ignored.
- * This function returns -1 in case of error.
- *****************************************************************************/
-static int ListenTCP( vlc_object_t * p_this, network_socket_t * p_socket )
-{
-    char * psz_server_addr = p_socket->psz_server_addr;
-    int i_server_port = p_socket->i_server_port;
-
-    int i_handle, i_dummy = 1;
-    struct sockaddr_in sock;
-
-    if( (i_handle = SocketTCP( p_this )) == -1 )
-        return VLC_EGENERIC;
-
-    if ( setsockopt( i_handle, SOL_SOCKET, SO_REUSEADDR,
-                (void *)&i_dummy, sizeof( i_dummy ) ) == -1 )
-    {
-        msg_Warn( p_this, "cannot configure socket (SO_REUSEADDR)" );
-    }
-
-    /* Build remote address */
-    if ( BuildAddr( &sock, psz_server_addr, i_server_port ) == -1 )
-    {
-        msg_Dbg( p_this, "could not build local address" );
-        return VLC_EGENERIC;
-    }
-    
-    /* Bind the socket */
-    if( bind( i_handle, (struct sockaddr *) &sock, sizeof( sock )) == -1 )
-    {
-#if defined(WIN32) || defined(UNDER_CE)
-        msg_Err( p_this, "cannot bind socket (%i)", WSAGetLastError());
-#else
-        msg_Err( p_this, "cannot bind socket (%s)", strerror(errno) );
-#endif
-        goto error;
-    }
- 
-    /* Listen */
-    if( listen( i_handle, 100 ) == -1 )
-    {
-#if defined(WIN32) || defined(UNDER_CE)
-        msg_Err( p_this, "cannot bring socket in listening mode (%i)",
-                 WSAGetLastError());
-#else
-        msg_Err( p_this, "cannot bring the socket in listening mode (%s)",
-                 strerror(errno) );
-#endif
-        goto error;
-    }
-
-    p_socket->i_handle = i_handle;
-    p_socket->i_mtu = 0; /* There is no MTU notion in TCP */
-    return VLC_SUCCESS;
-
-error:
-    close( i_handle );
-    return VLC_EGENERIC;
-}
-
-/*****************************************************************************
- * NetOpen: wrapper around OpenUDP, ListenTCP and OpenTCP
- *****************************************************************************/
-static int NetOpen( vlc_object_t * p_this )
-{
-    network_socket_t * p_socket = p_this->p_private;
-
-    if( p_socket->i_type == NETWORK_UDP )
-    {
-        return OpenUDP( p_this, p_socket );
-    }
-    else if( p_socket->i_type == NETWORK_TCP_PASSIVE )
-    {
-        return ListenTCP( p_this, p_socket );
-    }
-    else
-    {
-        return OpenTCP( p_this, p_socket );
-    }
+    return 0;
 }
