@@ -1,8 +1,8 @@
 /*****************************************************************************
  * deinterlace.c : deinterlacer plugin for vlc
  *****************************************************************************
- * Copyright (C) 2000, 2001, 2002, 2003 VideoLAN
- * $Id: deinterlace.c 11257 2005-06-02 17:06:00Z fkuehne $
+ * Copyright (C) 2000, 2001, 2002, 2003 the VideoLAN team
+ * $Id: deinterlace.c 15030 2006-04-01 20:33:59Z sam $
  *
  * Author: Sam Hocevar <sam@zoy.org>
  *
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -72,7 +72,10 @@ static void MergeGeneric ( void *, const void *, const void *, size_t );
 static void MergeAltivec ( void *, const void *, const void *, size_t );
 #endif
 #if defined(CAN_COMPILE_MMXEXT)
-static void MergeMMX     ( void *, const void *, const void *, size_t );
+static void MergeMMXEXT  ( void *, const void *, const void *, size_t );
+#endif
+#if defined(CAN_COMPILE_3DNOW)
+static void Merge3DNow   ( void *, const void *, const void *, size_t );
 #endif
 #if defined(CAN_COMPILE_SSE)
 static void MergeSSE2    ( void *, const void *, const void *, size_t );
@@ -80,6 +83,9 @@ static void MergeSSE2    ( void *, const void *, const void *, size_t );
 #if defined(CAN_COMPILE_MMXEXT) || defined(CAN_COMPILE_SSE)
 static void EndMMX       ( void );
 #endif
+#if defined(CAN_COMPILE_3DNOW)
+static void End3DNow     ( void );
+#endif
 
 static int  SendEvents   ( vlc_object_t *, char const *,
                            vlc_value_t, vlc_value_t, void * );
@@ -100,7 +106,10 @@ static int FilterCallback ( vlc_object_t *, char const *,
  * Module descriptor
  *****************************************************************************/
 #define MODE_TEXT N_("Deinterlace mode")
-#define MODE_LONGTEXT N_("You can choose the default deinterlace mode")
+#define MODE_LONGTEXT N_("Deinterlace method to use for local playback.")
+
+#define SOUT_MODE_TEXT N_("Streaming deinterlace mode")
+#define SOUT_MODE_LONGTEXT N_("Deinterlace method to use for streaming.")
 
 #define FILTER_CFG_PREFIX "sout-deinterlace-"
 
@@ -115,6 +124,7 @@ vlc_module_begin();
     set_category( CAT_VIDEO );
     set_subcategory( SUBCAT_VIDEO_VFILTER );
 
+    set_section( N_("Display"),NULL);
     add_string( "deinterlace-mode", "discard", NULL, MODE_TEXT,
                 MODE_LONGTEXT, VLC_FALSE );
         change_string_list( mode_list, mode_list_text, 0 );
@@ -124,8 +134,9 @@ vlc_module_begin();
 
     add_submodule();
     set_capability( "video filter2", 0 );
-    add_string( FILTER_CFG_PREFIX "mode", "blend", NULL, MODE_TEXT,
-                MODE_LONGTEXT, VLC_FALSE );
+    set_section( N_("Streaming"),NULL);
+    add_string( FILTER_CFG_PREFIX "mode", "blend", NULL, SOUT_MODE_TEXT,
+                SOUT_MODE_LONGTEXT, VLC_FALSE );
         change_string_list( mode_list, mode_list_text, 0 );
     set_callbacks( OpenFilter, CloseFilter );
 vlc_module_end();
@@ -190,7 +201,7 @@ static int Create( vlc_object_t *p_this )
     p_vout->pf_control = Control;
 
     p_vout->p_sys->i_mode = DEINTERLACE_DISCARD;
-    p_vout->p_sys->b_double_rate = 0;
+    p_vout->p_sys->b_double_rate = VLC_FALSE;
     p_vout->p_sys->last_date = 0;
     p_vout->p_sys->p_vout = 0;
     vlc_mutex_init( p_vout, &p_vout->p_sys->filter_lock );
@@ -212,12 +223,20 @@ static int Create( vlc_object_t *p_this )
     else
 #endif
 #if defined(CAN_COMPILE_MMXEXT)
-    if( p_vout->p_libvlc->i_cpu & CPU_CAPABILITY_MMX )
+    if( p_vout->p_libvlc->i_cpu & CPU_CAPABILITY_MMXEXT )
     {
-        p_vout->p_sys->pf_merge = MergeMMX;
+        p_vout->p_sys->pf_merge = MergeMMXEXT;
         p_vout->p_sys->pf_end_merge = EndMMX;
     }
     else
+#endif
+#if defined(CAN_COMPILE_3DNOW)
+    if( p_vout->p_libvlc->i_cpu & CPU_CAPABILITY_3DNOW )
+    {
+        p_vout->p_sys->pf_merge = Merge3DNow;
+        p_vout->p_sys->pf_end_merge = End3DNow;
+    }
+    else
 #endif
     {
         p_vout->p_sys->pf_merge = MergeGeneric;
@@ -253,35 +272,35 @@ static void SetFilterMethod( vout_thread_t *p_vout, char *psz_method )
     if( !strcmp( psz_method, "discard" ) )
     {
         p_vout->p_sys->i_mode = DEINTERLACE_DISCARD;
-        p_vout->p_sys->b_double_rate = 0;
+        p_vout->p_sys->b_double_rate = VLC_FALSE;
     }
     else if( !strcmp( psz_method, "mean" ) )
     {
         p_vout->p_sys->i_mode = DEINTERLACE_MEAN;
-        p_vout->p_sys->b_double_rate = 0;
+        p_vout->p_sys->b_double_rate = VLC_FALSE;
     }
     else if( !strcmp( psz_method, "blend" )
              || !strcmp( psz_method, "average" )
              || !strcmp( psz_method, "combine-fields" ) )
     {
         p_vout->p_sys->i_mode = DEINTERLACE_BLEND;
-        p_vout->p_sys->b_double_rate = 0;
+        p_vout->p_sys->b_double_rate = VLC_FALSE;
     }
     else if( !strcmp( psz_method, "bob" )
              || !strcmp( psz_method, "progressive-scan" ) )
     {
         p_vout->p_sys->i_mode = DEINTERLACE_BOB;
-        p_vout->p_sys->b_double_rate = 1;
+        p_vout->p_sys->b_double_rate = VLC_TRUE;
     }
     else if( !strcmp( psz_method, "linear" ) )
     {
         p_vout->p_sys->i_mode = DEINTERLACE_LINEAR;
-        p_vout->p_sys->b_double_rate = 1;
+        p_vout->p_sys->b_double_rate = VLC_TRUE;
     }
     else if( !strcmp( psz_method, "x" ) )
     {
         p_vout->p_sys->i_mode = DEINTERLACE_X;
-        p_vout->p_sys->b_double_rate = 0;
+        p_vout->p_sys->b_double_rate = VLC_FALSE;
     }
     else
     {
@@ -314,6 +333,7 @@ static int Init( vout_thread_t *p_vout )
             p_vout->output.i_width  = p_vout->render.i_width;
             p_vout->output.i_height = p_vout->render.i_height;
             p_vout->output.i_aspect = p_vout->render.i_aspect;
+            p_vout->fmt_out = p_vout->fmt_in;
             break;
 
         default:
@@ -353,13 +373,7 @@ static vout_thread_t *SpawnRealVout( vout_thread_t *p_vout )
 
     msg_Dbg( p_vout, "spawning the real video output" );
 
-    fmt.i_width = fmt.i_visible_width = p_vout->output.i_width;
-    fmt.i_height = fmt.i_visible_height = p_vout->output.i_height;
-    fmt.i_x_offset = fmt.i_y_offset = 0;
-    fmt.i_chroma = p_vout->output.i_chroma;
-    fmt.i_aspect = p_vout->output.i_aspect;
-    fmt.i_sar_num = p_vout->output.i_aspect * fmt.i_height / fmt.i_width;
-    fmt.i_sar_den = VOUT_ASPECT_FACTOR;
+    fmt = p_vout->fmt_out;
 
     switch( p_vout->render.i_chroma )
     {
@@ -370,7 +384,8 @@ static vout_thread_t *SpawnRealVout( vout_thread_t *p_vout )
         {
         case DEINTERLACE_MEAN:
         case DEINTERLACE_DISCARD:
-            fmt.i_height = fmt.i_visible_height = p_vout->output.i_height / 2;
+            fmt.i_height /= 2; fmt.i_visible_height /= 2; fmt.i_y_offset /= 2;
+            fmt.i_sar_den *= 2;
             p_real_vout = vout_Create( p_vout, &fmt );
             break;
 
@@ -440,13 +455,32 @@ static void Destroy( vlc_object_t *p_this )
  *****************************************************************************/
 static void Render ( vout_thread_t *p_vout, picture_t *p_pic )
 {
+    vout_sys_t *p_sys = p_vout->p_sys;
     picture_t *pp_outpic[2];
 
+    p_vout->fmt_out.i_x_offset = p_sys->p_vout->fmt_in.i_x_offset =
+        p_vout->fmt_in.i_x_offset;
+    p_vout->fmt_out.i_y_offset = p_sys->p_vout->fmt_in.i_y_offset =
+        p_vout->fmt_in.i_y_offset;
+    p_vout->fmt_out.i_visible_width = p_sys->p_vout->fmt_in.i_visible_width =
+        p_vout->fmt_in.i_visible_width;
+    p_vout->fmt_out.i_visible_height = p_sys->p_vout->fmt_in.i_visible_height =
+        p_vout->fmt_in.i_visible_height;
+    if( p_vout->p_sys->i_mode == DEINTERLACE_MEAN ||
+        p_vout->p_sys->i_mode == DEINTERLACE_DISCARD )
+    {
+        p_vout->fmt_out.i_y_offset /= 2; p_sys->p_vout->fmt_in.i_y_offset /= 2;
+        p_vout->fmt_out.i_visible_height /= 2;
+        p_sys->p_vout->fmt_in.i_visible_height /= 2;
+    }
+ 
+    pp_outpic[0] = pp_outpic[1] = NULL;
+
     vlc_mutex_lock( &p_vout->p_sys->filter_lock );
 
     /* Get a new picture */
     while( ( pp_outpic[0] = vout_CreatePicture( p_vout->p_sys->p_vout,
-                                             0, 0, 0 ) )
+                                                0, 0, 0 ) )
               == NULL )
     {
         if( p_vout->b_die || p_vout->b_error )
@@ -455,7 +489,7 @@ static void Render ( vout_thread_t *p_vout, picture_t *p_pic )
             return;
         }
         msleep( VOUT_OUTMEM_SLEEP );
-     }
+    }
 
     vout_DatePicture( p_vout->p_sys->p_vout, pp_outpic[0], p_pic->date );
 
@@ -560,7 +594,7 @@ static void RenderDiscard( vout_thread_t *p_vout,
                 p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                           p_pic->p[i_plane].i_pitch );
 
-                p_out += p_pic->p[i_plane].i_pitch;
+                p_out += p_outpic->p[i_plane].i_pitch;
                 p_in += 2 * p_pic->p[i_plane].i_pitch;
             }
             break;
@@ -575,10 +609,10 @@ static void RenderDiscard( vout_thread_t *p_vout,
                 {
                     p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                               p_pic->p[i_plane].i_pitch );
-                    p_out += p_pic->p[i_plane].i_pitch;
+                    p_out += p_outpic->p[i_plane].i_pitch;
                     p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                               p_pic->p[i_plane].i_pitch );
-                    p_out += p_pic->p[i_plane].i_pitch;
+                    p_out += p_outpic->p[i_plane].i_pitch;
                     p_in += i_increment;
                 }
             }
@@ -588,7 +622,7 @@ static void RenderDiscard( vout_thread_t *p_vout,
                 {
                     p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                               p_pic->p[i_plane].i_pitch );
-                    p_out += p_pic->p[i_plane].i_pitch;
+                    p_out += p_outpic->p[i_plane].i_pitch;
                     p_in += i_increment;
                 }
             }
@@ -629,7 +663,7 @@ static void RenderBob( vout_thread_t *p_vout,
                     p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                               p_pic->p[i_plane].i_pitch );
                     p_in += p_pic->p[i_plane].i_pitch;
-                    p_out += p_pic->p[i_plane].i_pitch;
+                    p_out += p_outpic->p[i_plane].i_pitch;
                 }
 
                 p_out_end -= 2 * p_outpic->p[i_plane].i_pitch;
@@ -639,13 +673,13 @@ static void RenderBob( vout_thread_t *p_vout,
                     p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                               p_pic->p[i_plane].i_pitch );
 
-                    p_out += p_pic->p[i_plane].i_pitch;
+                    p_out += p_outpic->p[i_plane].i_pitch;
 
                     p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                               p_pic->p[i_plane].i_pitch );
 
                     p_in += 2 * p_pic->p[i_plane].i_pitch;
-                    p_out += p_pic->p[i_plane].i_pitch;
+                    p_out += p_outpic->p[i_plane].i_pitch;
                 }
 
                 p_vout->p_vlc->pf_memcpy( p_out, p_in,
@@ -655,7 +689,7 @@ static void RenderBob( vout_thread_t *p_vout,
                 if( i_field == 0 )
                 {
                     p_in += p_pic->p[i_plane].i_pitch;
-                    p_out += p_pic->p[i_plane].i_pitch;
+                    p_out += p_outpic->p[i_plane].i_pitch;
                     p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                               p_pic->p[i_plane].i_pitch );
                 }
@@ -668,7 +702,7 @@ static void RenderBob( vout_thread_t *p_vout,
                     p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                               p_pic->p[i_plane].i_pitch );
                     p_in += p_pic->p[i_plane].i_pitch;
-                    p_out += p_pic->p[i_plane].i_pitch;
+                    p_out += p_outpic->p[i_plane].i_pitch;
                 }
 
                 p_out_end -= 2 * p_outpic->p[i_plane].i_pitch;
@@ -680,13 +714,13 @@ static void RenderBob( vout_thread_t *p_vout,
                         p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                                   p_pic->p[i_plane].i_pitch );
 
-                        p_out += p_pic->p[i_plane].i_pitch;
+                        p_out += p_outpic->p[i_plane].i_pitch;
 
                         p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                                   p_pic->p[i_plane].i_pitch );
 
                         p_in += 2 * p_pic->p[i_plane].i_pitch;
-                        p_out += p_pic->p[i_plane].i_pitch;
+                        p_out += p_outpic->p[i_plane].i_pitch;
                     }
                 }
                 else
@@ -696,7 +730,7 @@ static void RenderBob( vout_thread_t *p_vout,
                         p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                                   p_pic->p[i_plane].i_pitch );
 
-                        p_out += p_pic->p[i_plane].i_pitch;
+                        p_out += p_outpic->p[i_plane].i_pitch;
                         p_in += 2 * p_pic->p[i_plane].i_pitch;
                     }
                 }
@@ -708,7 +742,7 @@ static void RenderBob( vout_thread_t *p_vout,
                 if( i_field == 0 )
                 {
                     p_in += p_pic->p[i_plane].i_pitch;
-                    p_out += p_pic->p[i_plane].i_pitch;
+                    p_out += p_outpic->p[i_plane].i_pitch;
                     p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                               p_pic->p[i_plane].i_pitch );
                 }
@@ -744,7 +778,7 @@ static void RenderLinear( vout_thread_t *p_vout,
             p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                       p_pic->p[i_plane].i_pitch );
             p_in += p_pic->p[i_plane].i_pitch;
-            p_out += p_pic->p[i_plane].i_pitch;
+            p_out += p_outpic->p[i_plane].i_pitch;
         }
 
         p_out_end -= 2 * p_outpic->p[i_plane].i_pitch;
@@ -754,13 +788,13 @@ static void RenderLinear( vout_thread_t *p_vout,
             p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                       p_pic->p[i_plane].i_pitch );
 
-            p_out += p_pic->p[i_plane].i_pitch;
+            p_out += p_outpic->p[i_plane].i_pitch;
 
             Merge( p_out, p_in, p_in + 2 * p_pic->p[i_plane].i_pitch,
                    p_pic->p[i_plane].i_pitch );
 
             p_in += 2 * p_pic->p[i_plane].i_pitch;
-            p_out += p_pic->p[i_plane].i_pitch;
+            p_out += p_outpic->p[i_plane].i_pitch;
         }
 
         p_vout->p_vlc->pf_memcpy( p_out, p_in,
@@ -770,7 +804,7 @@ static void RenderLinear( vout_thread_t *p_vout,
         if( i_field == 0 )
         {
             p_in += p_pic->p[i_plane].i_pitch;
-            p_out += p_pic->p[i_plane].i_pitch;
+            p_out += p_outpic->p[i_plane].i_pitch;
             p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                       p_pic->p[i_plane].i_pitch );
         }
@@ -800,7 +834,7 @@ static void RenderMean( vout_thread_t *p_vout,
             Merge( p_out, p_in, p_in + p_pic->p[i_plane].i_pitch,
                    p_pic->p[i_plane].i_pitch );
 
-            p_out += p_pic->p[i_plane].i_pitch;
+            p_out += p_outpic->p[i_plane].i_pitch;
             p_in += 2 * p_pic->p[i_plane].i_pitch;
         }
     }
@@ -831,7 +865,7 @@ static void RenderBlend( vout_thread_t *p_vout,
                 /* First line: simple copy */
                 p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                           p_pic->p[i_plane].i_pitch );
-                p_out += p_pic->p[i_plane].i_pitch;
+                p_out += p_outpic->p[i_plane].i_pitch;
 
                 /* Remaining lines: mean value */
                 for( ; p_out < p_out_end ; )
@@ -839,7 +873,7 @@ static void RenderBlend( vout_thread_t *p_vout,
                     Merge( p_out, p_in, p_in + p_pic->p[i_plane].i_pitch,
                            p_pic->p[i_plane].i_pitch );
 
-                    p_out += p_pic->p[i_plane].i_pitch;
+                    p_out += p_outpic->p[i_plane].i_pitch;
                     p_in += p_pic->p[i_plane].i_pitch;
                 }
                 break;
@@ -848,7 +882,7 @@ static void RenderBlend( vout_thread_t *p_vout,
                 /* First line: simple copy */
                 p_vout->p_vlc->pf_memcpy( p_out, p_in,
                                           p_pic->p[i_plane].i_pitch );
-                p_out += p_pic->p[i_plane].i_pitch;
+                p_out += p_outpic->p[i_plane].i_pitch;
 
                 /* Remaining lines: mean value */
                 if( i_plane == Y_PLANE )
@@ -858,7 +892,7 @@ static void RenderBlend( vout_thread_t *p_vout,
                         Merge( p_out, p_in, p_in + p_pic->p[i_plane].i_pitch,
                                p_pic->p[i_plane].i_pitch );
 
-                        p_out += p_pic->p[i_plane].i_pitch;
+                        p_out += p_outpic->p[i_plane].i_pitch;
                         p_in += p_pic->p[i_plane].i_pitch;
                     }
                 }
@@ -870,7 +904,7 @@ static void RenderBlend( vout_thread_t *p_vout,
                         Merge( p_out, p_in, p_in + p_pic->p[i_plane].i_pitch,
                                p_pic->p[i_plane].i_pitch );
 
-                        p_out += p_pic->p[i_plane].i_pitch;
+                        p_out += p_outpic->p[i_plane].i_pitch;
                         p_in += 2*p_pic->p[i_plane].i_pitch;
                     }
                 }
@@ -911,8 +945,8 @@ static void MergeGeneric( void *_p_dest, const void *_p_s1,
 }
 
 #if defined(CAN_COMPILE_MMXEXT)
-static void MergeMMX( void *_p_dest, const void *_p_s1, const void *_p_s2,
-                      size_t i_bytes )
+static void MergeMMXEXT( void *_p_dest, const void *_p_s1, const void *_p_s2,
+                         size_t i_bytes )
 {
     uint8_t* p_dest = (uint8_t*)_p_dest;
     const uint8_t *p_s1 = (const uint8_t *)_p_s1;
@@ -939,6 +973,35 @@ static void MergeMMX( void *_p_dest, const void *_p_s1, const void *_p_s2,
 }
 #endif
 
+#if defined(CAN_COMPILE_3DNOW)
+static void Merge3DNow( void *_p_dest, const void *_p_s1, const void *_p_s2,
+                        size_t i_bytes )
+{
+    uint8_t* p_dest = (uint8_t*)_p_dest;
+    const uint8_t *p_s1 = (const uint8_t *)_p_s1;
+    const uint8_t *p_s2 = (const uint8_t *)_p_s2;
+    uint8_t* p_end = p_dest + i_bytes - 8;
+    while( p_dest < p_end )
+    {
+        __asm__  __volatile__( "movq %2,%%mm1;"
+                               "pavgusb %1, %%mm1;"
+                               "movq %%mm1, %0" :"=m" (*p_dest):
+                                                 "m" (*p_s1),
+                                                 "m" (*p_s2) );
+        p_dest += 8;
+        p_s1 += 8;
+        p_s2 += 8;
+    }
+
+    p_end += 8;
+
+    while( p_dest < p_end )
+    {
+        *p_dest++ = ( (uint16_t)(*p_s1++) + (uint16_t)(*p_s2++) ) >> 1;
+    }
+}
+#endif
+
 #if defined(CAN_COMPILE_SSE)
 static void MergeSSE2( void *_p_dest, const void *_p_s1, const void *_p_s2,
                        size_t i_bytes )
@@ -947,7 +1010,7 @@ static void MergeSSE2( void *_p_dest, const void *_p_s1, const void *_p_s2,
     const uint8_t *p_s1 = (const uint8_t *)_p_s1;
     const uint8_t *p_s2 = (const uint8_t *)_p_s2;
     uint8_t* p_end;
-    while( (int)p_s1 % 16 )
+    while( (ptrdiff_t)p_s1 % 16 )
     {
         *p_dest++ = ( (uint16_t)(*p_s1++) + (uint16_t)(*p_s2++) ) >> 1;
     }        
@@ -980,6 +1043,13 @@ static void EndMMX( void )
 }
 #endif
 
+#if defined(CAN_COMPILE_3DNOW)
+static void End3DNow( void )
+{
+    __asm__ __volatile__( "femms" :: );
+}
+#endif
+
 #ifdef CAN_COMPILE_C_ALTIVEC
 static void MergeAltivec( void *_p_dest, const void *_p_s1,
                           const void *_p_s2, size_t i_bytes )
@@ -1872,7 +1942,6 @@ static inline void XDeintBand8x8MMXEXT( uint8_t *dst, int i_dst,
 static void RenderX( vout_thread_t *p_vout,
                      picture_t *p_outpic, picture_t *p_pic )
 {
-    vout_sys_t *p_sys = p_vout->p_sys;
     int i_plane;
 
     /* Copy image and skip lines */
