@@ -2,7 +2,7 @@
  * osdmenu.c: osd filter module
  *****************************************************************************
  * Copyright (C) 2004-2005 M2X
- * $Id: osdmenu.c 11131 2005-05-23 11:04:07Z hartman $
+ * $Id: osdmenu.c 15199 2006-04-13 09:04:34Z zorglub $
  *
  * Authors: Jean-Paul Saman <jpsaman #_at_# m2x dot nl>
  *
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -39,37 +39,39 @@
  *****************************************************************************/
 
 /* FIXME: Future extension make the definition file in XML format. */
-#define OSD_FILE_TEXT N_("OSD menu configuration file")
+#define OSD_FILE_TEXT N_("Configuration file")
+/// \bug [String] missing dot
 #define OSD_FILE_LONGTEXT N_( \
-    "An OSD menu configuration file that menu actions with button images" )
-
+    "Configuration file for the OSD Menu" )
 #define OSD_PATH_TEXT N_("Path to OSD menu images")
 #define OSD_PATH_LONGTEXT N_( \
-    "Specify another path to the OSD menu images. This will override the path as defined in the " \
+    "Path to the OSD menu images. This will override the path defined in the " \
     "OSD configuration file." )
 
-#define POSX_TEXT N_("X coordinate of the OSD menu")
+#define POSX_TEXT N_("X coordinate")
 #define POSX_LONGTEXT N_("You can move the OSD menu by left-clicking on it." )
 
-#define POSY_TEXT N_("Y coordinate of the OSD menu")
+#define POSY_TEXT N_("Y coordinate")
 #define POSY_LONGTEXT N_("You can move the OSD menu by left-clicking on it." )
 
-#define POS_TEXT N_("OSD menu position")
+#define POS_TEXT N_("Menu position")
 #define POS_LONGTEXT N_( \
   "You can enforce the OSD menu position on the video " \
   "(0=center, 1=left, 2=right, 4=top, 8=bottom, you can " \
-  "also use combinations of these values).")
+  "also use combinations of these values, eg. 6 = top-right).")
 
-#define TIMEOUT_TEXT N_("Timeout of OSD menu")
+#define TIMEOUT_TEXT N_("Menu timeout")
 #define TIMEOUT_LONGTEXT N_( \
-    "OSD menu pictures get a default timeout of 15 seconds added to their remaining time." \
-    "This will ensure that they are at least the specified time visible.")
+    "OSD menu pictures get a default timeout of 15 seconds added to their " \
+    "remaining time. This will ensure that they are at least the specified " \
+    "time visible.")
 
-#define OSD_UPDATE_TEXT N_("Update speed of OSD menu")
+#define OSD_UPDATE_TEXT N_("Menu update interval" )
 #define OSD_UPDATE_LONGTEXT N_( \
-    "Update the OSD menu picture every 200ms (default). Shorten the update time for " \
-    "environments that experience transmissions errors. Be carefull with this option " \
-    "because encoding OSD menu pictures is very computing intensive. The range is 0 - 1000 ms." )
+    "The default is to update the OSD menu picture every 200 ms. Shorten the" \
+    " update time for environments that experience transmissions errors. " \
+    "Be careful with this option as encoding OSD menu pictures is very " \
+    "computing intensive. The range is 0 - 1000 ms." )
 
 static int pi_pos_values[] = { 0, 1, 2, 4, 8, 5, 6, 9, 10 };
 static char *ppsz_pos_descriptions[] =
@@ -94,7 +96,7 @@ static int OSDMenuVisibleEvent( vlc_object_t *, char const *,
 #endif
 
 #define OSD_UPDATE_MIN     0
-#define OSD_UPDATE_DEFAULT 200
+#define OSD_UPDATE_DEFAULT 0
 #define OSD_UPDATE_MAX     1000
 
 vlc_module_begin();
@@ -113,7 +115,7 @@ vlc_module_begin();
         OSD_UPDATE_LONGTEXT, VLC_TRUE );
 
     set_capability( "sub filter", 100 );
-    set_description( N_("On Screen Display menu subfilter") );
+    set_description( N_("On Screen Display menu") );
     set_shortname( N_("OSD menu") );
     add_shortcut( "osdmenu" );
 /*
