@@ -1,8 +1,8 @@
 /*****************************************************************************
  * transform.c : transform image module for vlc
  *****************************************************************************
- * Copyright (C) 2000-2004 VideoLAN
- * $Id: transform.c,v 1.18 2004/01/25 20:05:28 hartman Exp $
+ * Copyright (C) 2000-2004 the VideoLAN team
+ * $Id: transform.c 13905 2006-01-12 23:10:04Z dionoea $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -64,7 +64,10 @@ static char *type_list_text[] = { N_("Rotate by 90 degrees"),
 
 vlc_module_begin();
     set_description( _("Video transformation filter") );
+    set_shortname( N_("Transformation"));
     set_capability( "video filter", 0 );
+    set_category( CAT_VIDEO );
+    set_subcategory( SUBCAT_VIDEO_VFILTER );
 
     add_string( "transform-type", "90", NULL,
                           TYPE_TEXT, TYPE_LONGTEXT, VLC_FALSE);
@@ -87,6 +90,14 @@ struct vout_sys_t
     vout_thread_t *p_vout;
 };
 
+/*****************************************************************************
+ * Control: control facility for the vout (forwards to child vout)
+ *****************************************************************************/
+static int Control( vout_thread_t *p_vout, int i_query, va_list args )
+{
+    return vout_vaControl( p_vout->p_sys->p_vout, i_query, args );
+}
+
 /*****************************************************************************
  * Create: allocates Transform video thread output method
  *****************************************************************************
@@ -110,6 +121,7 @@ static int Create( vlc_object_t *p_this )
     p_vout->pf_manage = NULL;
     p_vout->pf_render = Render;
     p_vout->pf_display = NULL;
+    p_vout->pf_control = Control;
 
     /* Look what method was requested */
     psz_method = config_GetPsz( p_vout, "transform-type" );
@@ -168,6 +180,7 @@ static int Init( vout_thread_t *p_vout )
 {
     int i_index;
     picture_t *p_pic;
+    video_format_t fmt = {0};
 
     I_OUTPUTPICTURES = 0;
 
@@ -176,24 +189,33 @@ static int Init( vout_thread_t *p_vout )
     p_vout->output.i_width  = p_vout->render.i_width;
     p_vout->output.i_height = p_vout->render.i_height;
     p_vout->output.i_aspect = p_vout->render.i_aspect;
+    p_vout->fmt_out = p_vout->fmt_in;
+    fmt = p_vout->fmt_out;
 
     /* Try to open the real video output */
     msg_Dbg( p_vout, "spawning the real video output" );
 
     if( p_vout->p_sys->b_rotation )
     {
-        p_vout->p_sys->p_vout = vout_Create( p_vout,
-                           p_vout->render.i_height, p_vout->render.i_width,
-                           p_vout->render.i_chroma,
-                           (uint64_t)VOUT_ASPECT_FACTOR
-                            * (uint64_t)VOUT_ASPECT_FACTOR
-                            / (uint64_t)p_vout->render.i_aspect );
+        fmt.i_width = p_vout->fmt_out.i_height;
+        fmt.i_visible_width = p_vout->fmt_out.i_visible_height;
+        fmt.i_x_offset = p_vout->fmt_out.i_y_offset;
+
+        fmt.i_height = p_vout->fmt_out.i_width;
+        fmt.i_visible_height = p_vout->fmt_out.i_visible_width;
+        fmt.i_y_offset = p_vout->fmt_out.i_x_offset;
+
+        fmt.i_aspect = VOUT_ASPECT_FACTOR *
+            (uint64_t)VOUT_ASPECT_FACTOR / fmt.i_aspect;
+
+        fmt.i_sar_num = p_vout->fmt_out.i_sar_den;
+        fmt.i_sar_den = p_vout->fmt_out.i_sar_num;
+
+        p_vout->p_sys->p_vout = vout_Create( p_vout, &fmt );
     }
     else
     {
-        p_vout->p_sys->p_vout = vout_Create( p_vout,
-                           p_vout->render.i_width, p_vout->render.i_height,
-                           p_vout->render.i_chroma, p_vout->render.i_aspect );
+        p_vout->p_sys->p_vout = vout_Create( p_vout, &fmt );
     }
 
     /* Everything failed */
@@ -236,9 +258,12 @@ static void Destroy( vlc_object_t *p_this )
 {
     vout_thread_t *p_vout = (vout_thread_t *)p_this;
 
-    DEL_CALLBACKS( p_vout->p_sys->p_vout, SendEvents );
-    vlc_object_detach( p_vout->p_sys->p_vout );
-    vout_Destroy( p_vout->p_sys->p_vout );
+    if( p_vout->p_sys->p_vout )
+    {
+        DEL_CALLBACKS( p_vout->p_sys->p_vout, SendEvents );
+        vlc_object_detach( p_vout->p_sys->p_vout );
+        vout_Destroy( p_vout->p_sys->p_vout );
+    }
 
     DEL_PARENT_CALLBACKS( SendEventsToChild );
 
@@ -281,8 +306,9 @@ static void Render( vout_thread_t *p_vout, picture_t *p_pic )
                 uint8_t *p_in = p_pic->p[i_index].p_pixels;
 
                 uint8_t *p_out = p_outpic->p[i_index].p_pixels;
-                uint8_t *p_out_end = p_out + p_outpic->p[i_index].i_lines
-                                              * p_outpic->p[i_index].i_pitch;
+                uint8_t *p_out_end = p_out +
+                    p_outpic->p[i_index].i_visible_lines *
+                    p_outpic->p[i_index].i_pitch;
 
                 for( ; p_out < p_out_end ; )
                 {
@@ -290,7 +316,8 @@ static void Render( vout_thread_t *p_vout, picture_t *p_pic )
 
                     p_out_end -= p_outpic->p[i_index].i_pitch
                                   - p_outpic->p[i_index].i_visible_pitch;
-                    p_line_end = p_in + p_pic->p[i_index].i_lines * i_pitch;
+                    p_line_end = p_in + p_pic->p[i_index].i_visible_lines *
+                        i_pitch;
 
                     for( ; p_in < p_line_end ; )
                     {
@@ -307,7 +334,7 @@ static void Render( vout_thread_t *p_vout, picture_t *p_pic )
             for( i_index = 0 ; i_index < p_pic->i_planes ; i_index++ )
             {
                 uint8_t *p_in = p_pic->p[i_index].p_pixels;
-                uint8_t *p_in_end = p_in + p_pic->p[i_index].i_lines
+                uint8_t *p_in_end = p_in + p_pic->p[i_index].i_visible_lines
                                             * p_pic->p[i_index].i_pitch;
 
                 uint8_t *p_out = p_outpic->p[i_index].p_pixels;
@@ -338,14 +365,16 @@ static void Render( vout_thread_t *p_vout, picture_t *p_pic )
                 uint8_t *p_in = p_pic->p[i_index].p_pixels;
 
                 uint8_t *p_out = p_outpic->p[i_index].p_pixels;
-                uint8_t *p_out_end = p_out + p_outpic->p[i_index].i_lines
-                                              * p_outpic->p[i_index].i_pitch;
+                uint8_t *p_out_end = p_out +
+                    p_outpic->p[i_index].i_visible_lines *
+                    p_outpic->p[i_index].i_pitch;
 
                 for( ; p_out < p_out_end ; )
                 {
                     uint8_t *p_in_end;
 
-                    p_in_end = p_in + p_pic->p[i_index].i_lines * i_pitch;
+                    p_in_end = p_in + p_pic->p[i_index].i_visible_lines *
+                        i_pitch;
 
                     for( ; p_in < p_in_end ; )
                     {
@@ -364,7 +393,7 @@ static void Render( vout_thread_t *p_vout, picture_t *p_pic )
             for( i_index = 0 ; i_index < p_pic->i_planes ; i_index++ )
             {
                 uint8_t *p_in = p_pic->p[i_index].p_pixels;
-                uint8_t *p_in_end = p_in + p_pic->p[i_index].i_lines
+                uint8_t *p_in_end = p_in + p_pic->p[i_index].i_visible_lines
                                             * p_pic->p[i_index].i_pitch;
 
                 uint8_t *p_out = p_outpic->p[i_index].p_pixels;
@@ -383,7 +412,7 @@ static void Render( vout_thread_t *p_vout, picture_t *p_pic )
             for( i_index = 0 ; i_index < p_pic->i_planes ; i_index++ )
             {
                 uint8_t *p_in = p_pic->p[i_index].p_pixels;
-                uint8_t *p_in_end = p_in + p_pic->p[i_index].i_lines
+                uint8_t *p_in_end = p_in + p_pic->p[i_index].i_visible_lines
                                             * p_pic->p[i_index].i_pitch;
 
                 uint8_t *p_out = p_outpic->p[i_index].p_pixels;
