@@ -2,7 +2,7 @@
  * plugin.c:
  *****************************************************************************
  * Copyright (C) 2004 the VideoLAN team
- * $Id: plugin.c 11664 2005-07-09 06:17:09Z courmisch $
+ * $Id: plugin.c 14903 2006-03-24 11:05:28Z zorglub $
  *
  * Authors: Cyril Deguet <asmax@videolan.org>
  *          Implementation of the winamp plugin MilkDrop
@@ -21,7 +21,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -82,7 +82,7 @@ static int Open( vlc_object_t *p_this )
     if ( p_filter->input.i_format != VLC_FOURCC('f','l','3','2' )
          || p_filter->output.i_format != VLC_FOURCC('f','l','3','2') )
     {
-        msg_Warn( p_filter, "Bad input or output format" );
+        msg_Warn( p_filter, "bad input or output format" );
         return VLC_EGENERIC;
     }
     if ( !AOUT_FMTS_SIMILAR( &p_filter->input, &p_filter->output ) )
@@ -219,7 +219,7 @@ static void Thread( vlc_object_t *p_this )
         module_Need( p_thread->p_opengl, "opengl provider", NULL, 0 );
     if( p_thread->p_module == NULL )
     {
-        msg_Err( p_thread, "No OpenGL provider found" );
+        msg_Err( p_thread, "unable to initialize OpenGL" );
         vlc_object_detach( p_thread->p_opengl );
         vlc_object_destroy( p_thread->p_opengl );
         return;
