@@ -1,8 +1,8 @@
 /*****************************************************************************
  * vlcplugin.cpp: a VLC plugin for Mozilla
  *****************************************************************************
- * Copyright (C) 2002 VideoLAN
- * $Id: vlcplugin.cpp,v 1.4 2003/02/18 13:13:12 sam Exp $
+ * Copyright (C) 2002-2005 the VideoLAN team
+ * $Id: vlcplugin.cpp 15273 2006-04-19 09:13:26Z damienf $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -18,17 +18,29 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
+#include "config.h"
+
 #include <vlc/vlc.h>
 
+#ifdef HAVE_MOZILLA_CONFIG_H
+#   include <mozilla-config.h>
+#endif
+#include <nsISupports.h>
+#include <nsMemory.h>
 #include <npapi.h>
 
-#include "vlcpeer.h"
+#if !defined(XP_MACOSX) && !defined(XP_UNIX) && !defined(XP_WIN)
+#define XP_UNIX 1
+#elif defined(XP_MACOSX)
+#undef XP_UNIX
+#endif
+
 #include "vlcplugin.h"
 
 /*****************************************************************************
@@ -83,36 +95,3 @@ VlcIntf* VlcPlugin::GetPeer()
     return p_peer;
 }
 
-void VlcPlugin::SetFileName(const char * filename)
-{
-#if 0
-    FILE * fh;
-    fh = fopen(filename, "rb");
-    if(!fh)
-    {
-        fprintf(stderr, "Error while opening %s.\n", filename);
-        return;
-    }
-    fseek(fh, 0, SEEK_END);
-    m_lSize = ftell(fh);
-    m_szSound = (char*) malloc(m_lSize);
-    if(!m_szSound)
-    {
-        fprintf(stderr, "Error while allocating memory.\n");
-        fclose(fh);
-        return;
-    }
-    rewind(fh);
-    long pos = 0;
-    do
-    {
-        pos += fread(m_szSound + pos, 1, m_lSize - pos, fh);
-        fprintf(stderr, "pos = %d\n", pos);
-    }
-    while (pos < m_lSize -1);
-    fclose (fh);
-    fprintf(stderr, "File loaded\n");
-#endif
-    return;
-}
-
